@@ -84,21 +84,21 @@ int BinaryClock::getWidthFromHeight(int h) const
 
 void BinaryClock::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & Plasma::FormFactorConstraint) {
+    if (constraints & Plasma::SizeConstraint) {
         qreal top, bottom, left, right;
         getContentsMargins(&left, &top, &right, &bottom);
         qreal borderHeight = top + bottom;
         qreal borderWidth = left + right;
 
         if (formFactor() == Plasma::Vertical) {
-            setMaximumHeight(getHeightFromWidth((int) contentsRect().width()) + borderHeight);
-
+            setMinimumHeight(getHeightFromWidth((int) contentsRect().width()) + borderHeight);
+            setMinimumWidth(0);
         } else if (formFactor() == Plasma::Horizontal) {
-            setMaximumWidth(getWidthFromHeight((int) contentsRect().height()) + borderWidth);
-
+            setMinimumWidth(getWidthFromHeight((int) contentsRect().height()) + borderWidth);
+            setMinimumHeight(0);
         } else {
-            resize(getWidthFromHeight((int) contentsRect().height()) + borderWidth, contentsRect().height() + borderHeight);
-            emit appletTransformedItself();
+            setMinimumWidth(getWidthFromHeight(30));
+            setMinimumHeight(30);
         }
     }
 }
@@ -129,6 +129,7 @@ void BinaryClock::dataUpdated(const QString& source, const Plasma::DataEngine::D
     if (Plasma::ToolTipManager::self()->isVisible(this)) {
         updateTipContent();
     }
+    updateClockApplet();
 
     m_lastTimeSeen = m_time;
 
@@ -139,7 +140,7 @@ void BinaryClock::createClockConfigurationInterface(KConfigDialog *parent)
 {
     QWidget *widget = new QWidget();
     ui.setupUi(widget);
-    parent->addPage(widget, i18n("General"), icon());
+    parent->addPage(widget, i18n("Appearance"), "view-media-visualization");
 
     ui.showSecondHandCheckBox->setChecked(m_showSeconds);
     ui.showGridCheckBox->setChecked(m_showGrid);
@@ -269,7 +270,7 @@ void BinaryClock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *op
     int yPos = ((appletHeight - 4 * rectSize) / 2) + contentsRect.topLeft().y();
     int xPos = ((appletWidth - (rectSize * dots) - 5) / 2) + contentsRect.topLeft().x();
 
-    char timeDigits[6] = {m_time.hour() / 10, m_time.hour() % 10,
+    int timeDigits[6] = {m_time.hour() / 10, m_time.hour() % 10,
                           m_time.minute() / 10, m_time.minute() % 10,
                           m_time.second() / 10, m_time.second() % 10};
 
