@@ -29,14 +29,14 @@
 #include "picture.h"
 
 SlideShow::SlideShow(QObject *parent)
-    : QObject(parent)
+        : QObject(parent)
 {
     m_filters << "*.jpeg" << "*.jpg" << "*.png" << "*.svg" << "*.svgz" << "*.bmp" << "*.tif"; // use mime types?
     m_slideNumber = 0;
     m_useRandom = false;
 
     m_picture = new Picture(this);
-    connect(m_picture, SIGNAL(pictureLoaded(QPixmap)), this, SLOT(pictureLoaded(QPixmap)));
+    connect(m_picture, SIGNAL(pictureLoaded(QImage)), this, SLOT(pictureLoaded(QImage)));
 
     m_timer = new QTimer(this);
     connect(m_timer, SIGNAL(timeout()), this, SLOT(nextPicture()));
@@ -55,12 +55,12 @@ void SlideShow::setDirs(const QStringList &slideShowPath, bool recursive)
 {
     QDateTime setDirStart = QDateTime::currentDateTime();
 
-    m_image = QPixmap();
+    m_image = QImage();
     m_picturePaths.clear();
     foreach(const QString &path, slideShowPath) {
         addDir(KUrl(path).path(), recursive);
     }
-    
+
     KRandomSequence randomSequence;
     m_indexList.clear();
 
@@ -71,15 +71,14 @@ void SlideShow::setDirs(const QStringList &slideShowPath, bool recursive)
 
     // select 1st picture
     firstPicture();
-    
+
     // TODO do something meaningful if m_picturePaths.empty()
-    
     kDebug() << "Loaded " << m_picturePaths.size() << " pictures in " << setDirStart.secsTo(QDateTime::currentDateTime()) << " seconds";
 }
 
 void SlideShow::setImage(const QString &imagePath)
 {
-    m_image = QPixmap();
+    m_image = QImage();
     m_picturePaths.clear();
     addImage(imagePath);
     m_currentUrl = url();
@@ -99,24 +98,28 @@ void SlideShow::addDir(const QString &path, bool recursive)
         dirIterator.next();
         dirPicturePaths.append(dirIterator.filePath());
     }
-    
+
     // the pictures have to be sorted before adding them to the list,
     // because the QDirIterator sorts them in a different way than QDir::entryList
     dirPicturePaths.sort();
     m_picturePaths.append(dirPicturePaths);
 }
 
-QPixmap SlideShow::image() const 
+QImage SlideShow::image() const
 {
     if (m_image.isNull() || m_currentUrl != m_picture->url()) {
         kDebug() << "reloading from Picture" << m_currentUrl;
         //m_currentUrl = m_picture->url();
         m_picture->setPicture(m_currentUrl);
     }
-    kDebug();
     return m_image;
 }
 
+void SlideShow::updateImage(QString newUrl)
+{
+    m_picture->setPicture(newUrl);
+}
+
 KUrl SlideShow::url(int offset)
 {
     if (!m_picturePaths.isEmpty()) {
@@ -136,7 +139,6 @@ KUrl SlideShow::url(int offset)
             return KUrl(m_picturePaths.at(m_slideNumber));
         }
     }
-
     return KUrl();
 }
 
@@ -178,12 +180,12 @@ void SlideShow::setUpdateInterval(int msec)
     }
 }
 
-QString SlideShow::message() const 
+QString SlideShow::message() const
 {
     return m_picture->message();
 }
 
-void SlideShow::pictureLoaded(QPixmap image)
+void SlideShow::pictureLoaded(QImage image)
 {
     m_image = image;
     emit pictureUpdated();
@@ -191,21 +193,28 @@ void SlideShow::pictureLoaded(QPixmap image)
 
 void SlideShow::clearPicture()
 {
-    m_image = QPixmap();
+    m_image = QImage();
 }
 
 void SlideShow::dataUpdated(const QString &name, const Plasma::DataEngine::Data &data)
 {
     if (data.isEmpty()) {
-        m_image = QPixmap();
+        m_image = QImage();
         m_picture->setMessage(i18n("No Picture from this Provider."));
         return;
     }
 
-    m_image = data[name].value<QPixmap>();
+    m_image = data[name].value<QImage>();
+    //Compatibility with old dataengines
+    if (m_image.isNull()) {
+        QPixmap tmpPixmap = data[name].value<QPixmap>();
+        if (!tmpPixmap.isNull()) {
+            m_image = tmpPixmap.toImage();
+        }
+    }
+
     m_picture->setMessage(QString());
     emit pictureUpdated();
 }
 
 #include "slideshow.moc"
-
