@@ -30,14 +30,10 @@
 
 #include "ui_configuration.h"
 
-class KLineEdit;
-class KTextEdit;
-class KTextBrowser;
-class QSpinBox;
-class QCheckBox;
 class QGraphicsLinearLayout;
-class QGraphicsProxyWidget;
+class QGraphicsAnchorLayout;
 class KColorScheme;
+class PostWidget;
 
 namespace KWallet
 {
@@ -53,17 +49,10 @@ namespace Plasma
     class WebContent;
     class Frame;
     class ServiceJob;
-    class TextBrowser;
+    class TabBar;
     class ScrollWidget;
 }
 
-struct Tweet {
-    Plasma::Frame *frame;
-    Plasma::IconWidget *icon;
-    Plasma::TextBrowser *content;
-    Plasma::IconWidget *favIcon;
-};
-
 class MicroBlog : public Plasma::PopupApplet
 {
     Q_OBJECT
@@ -89,22 +78,23 @@ class MicroBlog : public Plasma::PopupApplet
          */
         void writeWallet(bool success);
 
-    protected slots:
+    protected Q_SLOTS:
         void configAccepted();
         void updateStatus();
         void updateCompleted(Plasma::ServiceJob *job);
         void downloadHistory();
-        void openProfile();
         void editTextChanged();
         void serviceFinished(Plasma::ServiceJob *job);
-        void updateSpinBoxSuffix();
 
         void themeChanged();
         void scheduleShowTweets();
         void showTweets();
+        void modeChanged(int index);
+        void openProfile(const QString &user = QString());
+        void reply(const QString &to);
+        void forward(const QString &rt);
 
     protected:
-        QString timeDescription( const QDateTime &dt );
         bool eventFilter(QObject *obj, QEvent *event);
         void popupEvent(bool show);
         void focusInEvent(QFocusEvent *event);
@@ -132,16 +122,22 @@ class MicroBlog : public Plasma::PopupApplet
          */
         void paintIcon();
 
+        /**
+         * Sets up the timeline service object (m_service)
+         */
+        void createTimelineService();
+
         Plasma::Svg *m_theme;
         Plasma::TextEdit *m_statusEdit;
         Plasma::WebContent *m_historyEdit;
         Plasma::FlashingLabel *m_flash;
         Plasma::IconWidget *m_icon;
         Plasma::ScrollWidget *m_scrollWidget;
+        Plasma::TabBar *m_tabBar;
         QGraphicsWidget *m_tweetsWidget;
         QGraphicsLinearLayout *m_tweetsLayout;
         QGraphicsLinearLayout *m_layout;
-        QGraphicsLinearLayout *m_headerLayout;
+        QGraphicsAnchorLayout *m_headerLayout;
         QGraphicsWidget *m_graphicsWidget;
         QPixmap m_popupIcon;
 
@@ -154,16 +150,19 @@ class MicroBlog : public Plasma::PopupApplet
         int m_historyRefresh;
         int m_newTweets;
         bool m_includeFriends;
+        int m_lastMode;
 
         Plasma::DataEngine *m_engine;
         Plasma::Service *m_service;
         Plasma::Service *m_profileService;
-        Plasma::Service *m_statusService;
+        QSet<Plasma::ServiceJob *> m_updateJobs;
         QString m_curTimeline;
+        Plasma::Frame *m_headerFrame;
 
         QMap<QString, QPixmap> m_pictureMap;
+        QStringList m_avatarHistory;
         QMap<qulonglong, Plasma::DataEngine::Data> m_tweetMap;
-        QList<Tweet> m_tweetWidgets;
+        QList<PostWidget *> m_tweetWidgets;
 
         qulonglong m_lastTweet;
         KWallet::Wallet *m_wallet;
@@ -173,7 +172,6 @@ class MicroBlog : public Plasma::PopupApplet
         KColorScheme *m_colorScheme;
         Ui::TwitterConfig configUi;
 
-        KTimeZone m_tz;
         QTimer *m_showTweetsTimer;
 };
 
