@@ -41,6 +41,8 @@ namespace Plasma
     class WeatherView;
 }
 
+class BackgroundWidget;
+
 class WeatherApplet : public WeatherPopupApplet
 {
     Q_OBJECT
@@ -58,6 +60,7 @@ public Q_SLOTS:
 protected Q_SLOTS:
     void configAccepted();
     void toolTipAboutToShow();
+    void clearCurrentWeatherIcon();
 
 protected:
     void constraintsEvent(Plasma::Constraints constraints);
@@ -72,9 +75,7 @@ private:
     Plasma::Label *m_locationLabel;
     Plasma::Label *m_forecastTemps;
     Plasma::Label *m_conditionsLabel;
-    Plasma::IconWidget *m_currentIcon;
     Plasma::Label *m_tempLabel;
-    Plasma::IconWidget *m_windIcon;
     Plasma::Label *m_courtesyLabel;
     Plasma::TabBar *m_tabBar;
 
@@ -88,8 +89,7 @@ private:
     int m_setupLayout;
     QGraphicsLinearLayout *m_layout;
     QGraphicsGridLayout *m_titlePanel;
-    QGraphicsLinearLayout *m_bottomLayout;
-    QGraphicsWidget *m_graphicsWidget;
+    BackgroundWidget *m_graphicsWidget;
 
     Plasma::Frame *m_titleFrame;
     QFont m_titleFont;
