@@ -1,4 +1,4 @@
-// Copyright 2008-2009 by Benoît Jacob <jacob.benoit.1@gmail.com>
+// Copyright 2008-2010 by Benoît Jacob <jacob.benoit.1@gmail.com>
 // 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -17,43 +17,64 @@
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "mandelbrot.h"
+#include <iostream>
 
 #include <cmath>
-#include <QPainter>
 #include <QGraphicsSceneMouseEvent>
+#include <kfiledialog.h>
+#include <QByteArray>
+#include <QBuffer>
+#include <kio/job.h>
 //#include <KDebug>
 
-#define MOUSE_MMB_SPEED ((qreal)10.)
-#define MOUSE_WHEEL_SPEED ((qreal)0.002)
+const qreal MOUSE_MMB_SPEED = (qreal)10.;
+const qreal MOUSE_WHEEL_SPEED = (qreal)0.002;
 
 K_EXPORT_PLASMA_WALLPAPER(mandelbrot, Mandelbrot)
 
 Mandelbrot::Mandelbrot(QObject *parent, const QVariantList &args)
-    : Plasma::Wallpaper(parent, args), m_image(0), m_tileImagePtr(0), m_tile(this),
+    : Plasma::Wallpaper(parent, args), m_image(0), m_tiling(this),
+      m_exportImageAction(i18n("Export Mandelbrot image..."), this),
+      m_exportConfigAction(i18n("Export Mandelbrot parameters..."), this),
+      m_importConfigAction(i18n("Import Mandelbrot parameters..."), this),
       m_abortRenderingAsSoonAsPossible(false),
-      m_imageIsReady(false)
+      m_imageIsReady(false),
+      m_firstInit(true)
 {
     qRegisterMetaType<MandelbrotTile>();
     m_hasSSE2 = system_has_SSE2();
     m_renderThreadCount = QThread::idealThreadCount();
     m_renderThreads = new MandelbrotRenderThread*[m_renderThreadCount];
-    for(int th = 0; th < m_renderThreadCount; th++) m_renderThreads[th] = new MandelbrotRenderThread(this, th);
+    for(int th = 0; th < m_renderThreadCount; th++) m_renderThreads[th] = new MandelbrotRenderThread(this);
     setUsingRenderingCache(true);
 
+    QList<QAction*> actionsList;
+    actionsList.append(&m_exportImageAction);
+    actionsList.append(&m_exportConfigAction);
+    actionsList.append(&m_importConfigAction);
+    setContextualActions(actionsList);
+
     connect(this, SIGNAL(renderHintsChanged()), this, SLOT(checkRenderHints()));
+    connect(&m_exportImageAction, SIGNAL(triggered()), this, SLOT(exportImage()));
+    connect(&m_exportConfigAction, SIGNAL(triggered()), this, SLOT(exportConfig()));
+    connect(&m_importConfigAction, SIGNAL(triggered()), this, SLOT(importConfig()));
 }
 
 Mandelbrot::~Mandelbrot()
 {
     abortRendering();
+    // the user may have moved the viewpoint, so we must save the config on exit (and that takes care of the cache, too)
+    emit(configNeedsSaving());
     for(int th = 0; th < m_renderThreadCount; th++) delete m_renderThreads[th];
     delete[] m_renderThreads;
     delete m_image;
-    delete m_tileImagePtr;
 }
 
 void Mandelbrot::updateCache()
 {
+    // Don't let the mini-previewer in Desktop Settings affect the cache.
+    if(isPreviewing()) return;
+    
     QString k = key();
 
     // if the view parameters changed since we loaded from cache, or if we couldn't load from cache
@@ -65,52 +86,56 @@ void Mandelbrot::updateCache()
 
         // if the image is ready to be cached, cache it
         if(m_imageIsReady) {
-            // only cache large images. Prevents the mini-monitor in Desktop Settings dialog from overwriting the cache
-            // (if we say, as we currently do, that the image size isn't written in the key) or from leaving stale cached images
-            // (if we made the image size part of the key).
-            if(width()*height()>100000) {
-                //kDebug() << "caching " << k << " replacing " << m_cacheKey;
-                insertIntoCache(k, *m_image);
-                m_cacheKey = k;
-            }
-            //else kDebug() << "small image, don't cache";
+            //kDebug() << "caching " << k << " replacing " << m_cacheKey;
+            insertIntoCache(k, *m_image);
+            m_cacheKey = k;
         }
     }
     //else kDebug() << k << " is already cached";
 }
 
 void Mandelbrot::paint(QPainter *painter, const QRectF& exposedRect)
-{   
+{
     painter->drawImage(exposedRect, *m_image, exposedRect.translated(-boundingRect().topLeft()));
 }
 
 void Mandelbrot::save(KConfigGroup &config)
 {
-    config.writeEntry(MANDELBROT_CENTER_KEY, m_center);
-    config.writeEntry(MANDELBROT_ZOOM_KEY, m_zoom);
+    if (!isPreviewing())
+    {
+        config.writeEntry(MANDELBROT_CENTER_KEY, m_center);
+        config.writeEntry(MANDELBROT_ZOOM_KEY, m_zoom);
+    }
+
     config.writeEntry(MANDELBROT_COLOR1_KEY, m_color1);
     config.writeEntry(MANDELBROT_COLOR2_KEY, m_color2);
     config.writeEntry(MANDELBROT_COLOR3_KEY, m_color3);
     config.writeEntry(MANDELBROT_QUALITY_KEY, m_quality);
     config.writeEntry(MANDELBROT_LOCK_KEY, int(m_lock));
-
     // and this is where we update the cache. It's important to update the cache at the same time we save the config.
     // Otherwise we could easily have stale cached images, or missing ones, for example in case of a crash.
     updateCache();
 }
 
-void Mandelbrot::init(const KConfigGroup &config)
+void Mandelbrot::readConfig(const KConfigGroup &config, int options)
 {
     QString old_key = key();
 
-    m_center = config.readEntry(MANDELBROT_CENTER_KEY, QPointF(0,0));
-    m_zoom = config.readEntry(MANDELBROT_ZOOM_KEY, qreal(2));
-    m_color1 = config.readEntry(MANDELBROT_COLOR1_KEY, QColor(255,255,255));
-    m_color2 = config.readEntry(MANDELBROT_COLOR2_KEY, QColor(64,255,255));
+    if(options & ReadViewpoint)
+    {
+        m_center = config.readEntry(MANDELBROT_CENTER_KEY, QPointF(qreal(-0.25),qreal(0)));
+        m_zoom = config.readEntry(MANDELBROT_ZOOM_KEY, qreal(4));
+    }
+
+    m_color1 = config.readEntry(MANDELBROT_COLOR1_KEY, QColor(0,0,0));
+    m_color2 = config.readEntry(MANDELBROT_COLOR2_KEY, QColor(255,255,255));
     m_color3 = config.readEntry(MANDELBROT_COLOR3_KEY, QColor(0,0,255));
-    m_quality = config.readEntry(MANDELBROT_QUALITY_KEY, 1);
-    if(m_quality > 4) m_quality = 4;
-    m_lock = Qt::CheckState(config.readEntry(MANDELBROT_LOCK_KEY, int(Qt::Unchecked)));
+    m_quality = qBound(0, config.readEntry(MANDELBROT_QUALITY_KEY, 1), 4);
+
+    if(options & ReadLockStatus)
+    {
+      m_lock = Qt::CheckState(config.readEntry(MANDELBROT_LOCK_KEY, int(Qt::Unchecked)));
+    }
 
     if(key() != old_key)
     {
@@ -120,12 +145,19 @@ void Mandelbrot::init(const KConfigGroup &config)
         {
             delete m_image;
             m_image = new QImage(width(), height(), MANDELBROT_QIMAGE_FORMAT);
+            QPainter(m_image).fillRect(m_image->rect(), Qt::black);
         }
 
         loadFromCacheOrStartRendering();
     }
 }
 
+void Mandelbrot::init(const KConfigGroup &config)
+{
+    readConfig(config, (m_firstInit?ReadViewpoint:0) | ReadLockStatus);
+    m_firstInit = false;
+}
+
 QWidget* Mandelbrot::createConfigurationInterface(QWidget* parent)
 {
     QWidget *widget = new QWidget(parent);
@@ -191,6 +223,7 @@ void Mandelbrot::checkRenderHints()
         abortRendering();
         delete m_image;
         m_image = new QImage(width(), height(), MANDELBROT_QIMAGE_FORMAT);
+        QPainter(m_image).fillRect(m_image->rect(), Qt::black);
         loadFromCacheOrStartRendering();
     }
 }
@@ -200,9 +233,17 @@ void Mandelbrot::mousePressEvent(QGraphicsSceneMouseEvent *event)
     event->ignore();
     if(m_lock) return;
     m_mousePressPos = m_mouseLastMovePos = event->pos();
+    m_mousePressedButtons = event->buttons();
     if(event->buttons() & (Qt::LeftButton|Qt::MidButton)) event->accept();
 }
 
+void Mandelbrot::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
+{
+    event->ignore();
+    if(m_lock) return;
+    if(m_mousePressedButtons & (Qt::LeftButton|Qt::MidButton)) event->accept();
+}
+
 void Mandelbrot::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
     event->ignore();
@@ -211,9 +252,9 @@ void Mandelbrot::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
     QPointF delta = event->pos() - m_mouseLastMovePos;
     m_mouseLastMovePos = event->pos();
-    
+
     if(event->buttons() & Qt::MidButton)
-    {      
+    {
         zoomView(m_mousePressPos, std::exp(MOUSE_MMB_SPEED * qreal(delta.y()) / height()));
     }
 
@@ -225,8 +266,8 @@ void Mandelbrot::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
 void Mandelbrot::wheelEvent(QGraphicsSceneWheelEvent *event)
 {
+    event->ignore();
     if(m_lock) return;
-
     event->accept();
     zoomView(event->pos(), std::exp(-MOUSE_WHEEL_SPEED * event->delta()));
 }
@@ -265,13 +306,21 @@ void Mandelbrot::loadFromCacheOrStartRendering()
 
 void Mandelbrot::startRendering(const QPointF& renderFirst)
 {
+    abortRendering();
     if(m_image->size() != boundingRect().size()) {
         delete m_image;
         m_image = new QImage(width(), height(), MANDELBROT_QIMAGE_FORMAT);
+        QPainter(m_image).fillRect(m_image->rect(), Qt::black);
     }
     m_imageIsReady = false;
-    m_tile.start(renderFirst);
-    m_renderThreads[0]->start(QThread::LowPriority);
+    m_tilesFinishedRendering = 0;
+    m_tiling.start(renderFirst);
+    computeStats();
+    // abort if required
+    if(abortRenderingAsSoonAsPossible()) return;
+    for(int i = 0; i < renderThreadCount(); i++) {
+        renderThread(i)->start(QThread::LowestPriority);
+    }
 }
 
 void Mandelbrot::translateView(const QPointF& _delta)
@@ -296,17 +345,16 @@ void Mandelbrot::translateView(const QPointF& _delta)
 
     // compute which pixel to render first, so we start with the tiles the user is most interested in
     qreal renderfirstx, renderfirsty;
-    int noise = (qrand()%(width()/2)) - width()/4; // if user is scrolling continuously, don't always rerender the same tile.
     if(delta.y()!=0 && qAbs(_delta.x()/_delta.y())<qAbs(qreal(width())/height()))
     {
       if(delta.y()>0)
       {
-        renderfirstx = width()/2 - _delta.x() * height() / (2 * _delta.y()) + noise;
+        renderfirstx = width()/2 - _delta.x() * height() / (2 * _delta.y());
         renderfirsty = 0.;
       }
       else
       {
-        renderfirstx = width()/2 + _delta.x() * height() / (2 * _delta.y()) + noise;
+        renderfirstx = width()/2 + _delta.x() * height() / (2 * _delta.y());
         renderfirsty = height();
       }
     }
@@ -314,12 +362,12 @@ void Mandelbrot::translateView(const QPointF& _delta)
     {
       if(delta.x()>0)
       {
-        renderfirsty = height()/2 - _delta.y() * width() / (2 * _delta.x()) + noise;
+        renderfirsty = height()/2 - _delta.y() * width() / (2 * _delta.x());
         renderfirstx = 0.;
       }
       else
       {
-        renderfirsty = height()/2 + _delta.y() * width() / (2 * _delta.x()) + noise;
+        renderfirsty = height()/2 + _delta.y() * width() / (2 * _delta.x());
         renderfirstx = width();
       }
     }
@@ -371,13 +419,6 @@ void Mandelbrot::zoomView(const QPointF& at, qreal zoomFactor)
     startRendering(at);
 }
 
-float Mandelbrot::gamma() const
-{
-    if(quality() == 0) return 0.40f;
-    else if(quality() == 1) return 0.28f;
-    else return 0.22f;
-}
-
 int Mandelbrot::maxIter() const
 {
     int max_iter_factor;
@@ -446,6 +487,54 @@ void Mandelbrot::computeStats()
 void Mandelbrot::tileDone(const MandelbrotTile& t)
 {
     emit update(QRectF(t.destination()).translated(boundingRect().topLeft()));
+    m_tilesFinishedRendering++;
+    if(m_tilesFinishedRendering >= TILING_SIZE*TILING_SIZE) m_imageIsReady = true;
 }
 
+void Mandelbrot::exportImage()
+{
+    KUrl url = KFileDialog::getSaveUrl(
+                   KUrl(),
+                   "*.png|PNG images",
+                   0,
+                   QString(),
+                   KFileDialog::ConfirmOverwrite
+                 );
+    QByteArray ba;
+    QBuffer buffer(&ba);
+    buffer.open(QIODevice::WriteOnly);
+    m_image->save(&buffer, "PNG");
+    KIO::file_delete(url);
+    KIO::storedPut(ba, url, -1);
+}
+
+void Mandelbrot::exportConfig()
+{
+    QString file = KFileDialog::getSaveFileName(
+                     KUrl(),
+                     "*.txt|Text files",
+                     0,
+                     QString(),
+                     KFileDialog::ConfirmOverwrite
+                   );
+    KConfig config(file, KConfig::SimpleConfig);
+    KConfigGroup configgroup(&config, "Mandelbrot");
+    save(configgroup);
+    configgroup.config()->sync();
+}
+
+void Mandelbrot::importConfig()
+{
+    QString file = KFileDialog::getOpenFileName(
+                     KUrl(),
+                     "*.txt|Text files",
+                     0,
+                     QString()
+                   );
+    if(file.isEmpty()) return;
+    KConfig config(file, KConfig::SimpleConfig);
+    KConfigGroup configgroup(&config, "Mandelbrot");
+    readConfig(configgroup, ReadViewpoint); // reading colors and quality level is implicit
+}                                           // but we dont want to read the lock status
+
 #include "mandelbrot.moc"
